//depend on Bittian.java
//depended by PointList

public class Point {
	double x;
	double y;
	double z;
	//--------------------------------------------------------------------------------
	//construct
	//--------------------------------------------------------------------------------
	public Point() {
		this.x = 0;
		this.y = 0;
		this.z = 0;
	}
	public Point(double x, double y, double z) {
		this.x = x;
		this.y = y;
		this.z = z;
	}
	public Point(Point p) {
		this.x = p.x;
		this.y = p.y;
		this.z = p.z;
	}
	//avoid multiple memory allocation if a point can be just recycled
	public Point set(Point p) {
		this.x = p.x;
		this.y = p.y;
		this.z = p.z;
		return this;
	}
	public Point set(double x, double y, double z) {
		this.x = x;
		this.y = y;
		this.z = z;
		return this;
	}
	//--------------------------------------------------------------------------------
	//operation
	//--------------------------------------------------------------------------------
	public Point add(double x, double y, double z) {
		//ideal for move
		this.x += x;
		this.y += y;
		this.z += z;
		return this;
	}
	public Point add(Point p) {
		//ideal for translation of polypoint to center point
		this.x += p.x;
		this.y += p.y;
		this.z += p.z;
		return this;
	}
	public Point sub(Point p) {
		//ideal for reverse_translation of polypoint to center point
		this.x -= p.x;
		this.y -= p.y;
		this.z -= p.z;
		return this;
	}
	public Point mul(double m) {
		//ideal for zoom
		this.x *= m;
		this.y *= m;
		this.z *= m;
		return this;
	}
	public Point mul(Point p) {
		this.x *= p.x;
		this.y *= p.y;
		this.z *= p.z;
		return this;
	}
	public Point mul(double x, double y, double z) {
		this.x *= x;
		this.y *= y;
		this.z *= z;
		return this;
	}
	public Point div(double d) {
		this.x /= d;
		this.y /= d;
		this.z /= d;
		return this;	
	}
	public Point div(Point p) {
		this.x /= p.x;
		this.y /= p.y;
		this.z /= p.z;	
		return this;
	}
	public Point div(double x, double y, double z) {
		this.x /= x;
		this.y /= y;
		this.z /= z;	
		return this;
	}
	//--------------------------------------------------------------------------------
	//it is like 3d hypothenuse but without sqrt
 	//--------------------------------------------------------------------------------
	public double dist() { 
		//will make easy backface culling and zsort when point are first traducted
		return (this.x*this.x)+(this.y*this.y)+(this.z*this.z);
	}
	public double dist(int x, int y, int z) { 
		//will make easy backface culling and zsort when point are first traducted
		x -= this.x;
		y -= this.y;
		z -= this.z;
		return (x*x)+(y*y)+(z*z);
	}
	public double dist(Point p) { 
		//will make easy backface culling and zsort when point are first traducted
		double x = p.x - this.x;
		double y = p.y - this.y;
		double z = p.z - this.z;
		return (x*x)+(y*y)+(z*z);
	}
	//--------------------------------------------------------------------------------
	//dot product   (A * B)  
 	//--------------------------------------------------------------------------------
	public double dot_product(double x, double y, double z) {
		//dot product from x,y,z
		return (this.x*x)+(this.y*y)+(this.z*z);
	}
	public double dot_product(Point p) {
		//dot_product from point
		return (this.x*p.x)+(this.y*p.y)+(this.z*p.z);
	}
	//a dot product for moving object direction can eleminate many collision detection
	//--------------------------------------------------------------------------------
	//cross product  (A x B)     return a vector perpendicular of two vector
	//--------------------------------------------------------------------------------
	public Point cross_product(double x, double y, double z) {
		//dont allow point to spawn other , make only self alteration
		return this.set( (this.y*z)-(this.z*y), (this.z*x)-(this.x*z), (this.x*y)-(this.y*x));
	}
	public Point cross_product(Point p) {
		//dont allow point to spawn other , make only self alteration
		return this.set( (this.y*p.z)-(this.z*p.y), (this.z*p.x)-(this.x*p.z), (this.x*p.y)-(this.y*p.x));
	}
	//--------------------------------------------------------------------------------
	//plane		the plane is a direction perpendicular to the polygon going to the infinite , where k is the position on that line
	//--------------------------------------------------------------------------------
	//calcutation the normal of three point (p1,p2,p3), is making the cross product of two line (p1,p2 and p2,p3)
	//normal = cross_product((p1-p2),(p3-p2))
	//k = dot_product(p1,normal);
	//to know if the point is in plane;
	//dot_product(point , normal) = k           Point is in plane 
	//dot_product(point , normal) > k           Point is on one side of plane 
	//dot_product(point , normal) < k           Point is on other side of plane
	// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
	//--------------------------------------------------------------------------------
	//rotate
	//--------------------------------------------------------------------------------
	public void rotate_x(double r) {
		double yy = this.y, zz = this.z, sin = Math.sin(r), cos = Math.cos(r);
		this.y = (yy*cos)-(zz*sin);
		this.z = (yy*sin)+(zz*cos);
	}
	public void rotate_y(double r) {
		double xx = this.x, zz = this.z, sin = Math.sin(r), cos = Math.cos(r);
		this.x = (zz*sin)+(xx*cos);
		this.z = (zz*cos)-(xx*sin);
	}
	public void rotate_z(double r) {
		double xx = this.x, yy = this.y, sin = Math.sin(r), cos = Math.cos(r);
		this.x = ((yy*sin)+(xx*cos));
		this.y = ((yy*cos)-(xx*sin));
	}
	//--------------------------------------------------------------------------------
	//
	//--------------------------------------------------------------------------------
}
